#pragma warning (disable: 4786)

#include <sstream>
#include <cstdlib>
#include <cassert>
#include "AsciiFile.h"

AsciiFile::AsciiFile(char* filename,char *blanks,char *fldseps,char  *recseps, bool ld, bool ir)
: File(filename, ld)
{
	isrec    = ir;
	bufsid   = -1;
	tfscan.tfs_chars(TFS_BLANK,  blanks);
	tfscan.tfs_chars(TFS_FLDSEP, fldseps);
	tfscan.tfs_chars(TFS_RECSEP, recseps);
	read_blk();
}


int AsciiFile::get_item()
{                                        /* --- read an item                  */
	ITEM *item;                          /* pointer to item                   */
	int  d;                              /* delimiter type                    */
	assert(file);                        /* checks some variables             */

     d = tfscan.tfs_getfld(file, buf, BUFSIZE-1, NULL);

	if (d == TFS_REC) 
		reccnt++;                       /* read next name from file           */
	if ((d < 0) || (buf[0] == '\0')) 
		return d;

	item = (ITEM*)nimap.lookup(std::string(buf));    /* look up name in name/id map        */
  
	if (item)                           /* if item found,                     */
		item->frq++;                    /* update item occurence              */
	else {                              /* if item not found                  */
		item = (ITEM*)nimap.insert(std::string(buf), sizeof(ITEM));
		if (!item) 
			return E_NOMEM;             /* add the new item to the map,       */
		item->frq = 1;                  /* initialize the frequency counter,  */
		//    item->app = dfltapp;      /* and set the appearance indicator   */
	}
	//cis.push_back(item->id);            /* add new item to item set           */
	cis.push_back(item);                /* add new item to item set           */
	return d;                           /* and return delimiter type          */
}  /* get_item() */                    



int AsciiFile::get_iset ()
{                                      /* --- read an item set                */
	int d, e,trans_id;                          /* delimiter type                      */
	ITEM* transaction;
	assert(file);                      /* checks some global variables        */
	cis.clear();                       /* start a new item set                */
									   /* by default, no duplicates           */
	/* --- one item set per record ---     */
	if (isrec) {                       /* if one item set per record,         */
		do { d = get_item();      /* read the items                      */
		} while ((d == TFS_FLD)        /* of the item set                     */
			&&       (buf[0] != '\0'));/* up to the end of the record         */
		
		if (d < 0) 
			return d;                  /* check for a read error              */
		if (cis.size() > 0) 
		{
			trans_id=(*this).size();
			std::stringstream ss;
			ss << trans_id;
			transaction = (ITEM*)trmap.lookup(ss.str());    /* look up name in transaction map        */
  				if (!transaction)              
				{	
					transaction = (ITEM*)trmap.insert(ss.str(), sizeof(ITEM));
				};
			return (buf[0] == '\0') ? E_ITEMEXP : TFS_REC;
		}
		return d;                      /* check for an empty item             */
	}                                  /* and return delimiter type           */
	/* --- set id and item per record ---  */
	if (bufsid < 0) 
		sid[0] = '\0';                 /* clear set identifier buffer         */
	e = TFS_EOF;                       /* init. end of file indicator         */
	while (1) {                        /* read loop for items                 */
		if (bufsid >= 0) {             /* if there is a  buffered set id,     */
			d = bufsid; 
			bufsid = -1;
		}                              /* get delimiter and clear flag        */
		else {                         /* if there is no buffered set id,     */
			do {                       /* read next set identifier            */
				d = tfscan.tfs_getfld(file, buf, BUFSIZE-1, NULL);
				if (d == TFS_REC) 
					reccnt++;          /* count line                          */
			} while ((d == TFS_REC)    /* skip empty lines                    */
				&&       (buf[0] == '\0'));
			if (d < 0) 
				return d;              /* check for a read error              */
			if (buf[0] == '\0') {      /* if no set identifier read           */
				if (d != TFS_EOF) 
					return E_SIDEXP;
				return (cis.size() > 0) ? TFS_REC : e;
			}                          /* check for end of file/end of set    */
			if (strcmp(buf, sid) !=0){ /* if a new set id has been found      */
				if (sid[0] == '\0')   /* if it is the first set id,          */
				{
					strcpy(sid, buf);  /* just copy it to the buffer          */
					transaction = (ITEM*)trmap.lookup(std::string(sid));    /* look up name in transaction map        */
  					if (!transaction)              
					{	
						transaction = (ITEM*)trmap.insert(std::string(sid), sizeof(ITEM));
					};
				}
				else {                 /* if it is not the first set id       */
					strcpy(sid, buf); 
					bufsid = d; 
					transaction = (ITEM*)trmap.lookup(std::string(sid));    /* look up name in transaction map        */
  					if (!transaction)              
					{	
						transaction = (ITEM*)trmap.insert(std::string(sid), sizeof(ITEM));
					};
					return TFS_REC; 
				};
			}                          /* copy it to the buffer, set flag,    */
		}                              /* and abort the function              */
		if (d < TFS_FLD) {             /* if there is no item,                */
			e = TFS_REC; 
			continue; 
		}                              /* clear end of file indicator         */
		d = get_item();                /* read item and check result          */
		if (d < 0)          return d;
		if (buf[0] == '\0') return E_ITEMEXP;
	//	if (d > TFS_REC)    return E_FLDCNT;
				if (d > TFS_REC)    error(E_FLDCNT,fn_in,reccnt);
	}  /* while(1) */
}  /* get_iset() */




void AsciiFile::read_blk()
{
	int     d;
	int     skip     = 0;         /* number of records to skip          */
	int     isrec    = 0;       
	
	
	file = fopen(fn_in, "r");     /* open input file for reading        */
	if (!file) 
		error(E_FOPEN,fn_in);
	
	for (reccnt = d = 0; reccnt < skip; reccnt++) {
		do {                         /* read fields from file           */
			d = tfscan.tfs_getfld(file, buf, BUFSIZE-1, NULL);
		} while (d == TFS_FLD);     /* while not at end of record       */
		if (d <= TFS_EOF) break;    /* check for read error/end of file */
	}                               /* (skip `skip' records)            */
	if (d < 0) 
		error(E_FREAD,fn_in);
	pos = ftell(file);              /* note the position reached        */
	if (pos < 0) 
		error(E_FREAD,fn_in);
	while (1) {                     /* read the item sets               */
		d = get_iset();             /* get the next item set and        */
		if (d <= TFS_EOF) {         /* check for read error/end of file */
			if (d < 0) 
				error(d); 
			else break; 
		}
		reccnt++;					/* increments the counter of records*/
		cis.prepare();              /* sort items and remove duplicates */
		if (cis.size() > maxcnt)    /* determine the maximum number     */
			maxcnt = cis.size();    /* of items in one item set         */
		isetsz++;
		if (load) {                 /* if to load item sets into memory */
			isets.push_back(cis);   /* store new item set               */
		}                           /* in the item set vector           */
	}

	/*
    for (int i = 0; i < items(); i++) {
		cout << i << "\t" << lookup(i) << endl;
	}
	cout << endl;
	for (i = 0; i < isets.size(); i++) {
		cout << "Trans: " << looktr(i) << "\t";
		for (int j = 0; j < isets[i].size(); j++) {
			//cout << lookup(isets[i][j]) << "(" <<isets[i][j] << ") ";
			//cout << isets[i][j].id << " ";
			cout << isets[i][j]->id << " ";
		}
		cout << endl;
	}
	/**/

	if (load)                       /* if the file is loaded in memory  */
		int ciao = 10;//ATTENZIONE_MODIFICA fclose(file);               /* we don't need the file anymore   */
	else                            /* otherwise reset file position    */  
		if(fseek(file, pos, SEEK_SET) != 0)
			error(E_FREAD,fn_in);   /* for further readings             */	
}

