#include "ClusterGamma.h"

double ClusterGamma::distance(ITEMSET& e)/* compute the distance w.r.t. the cluster representative */
{       
    int i;
    int j;                                  /* itemset indexes                                        */
    int sizec = representative.size();      /* the size of the cluster representative                 */
    int sizee = e.size();                   /* the size of the current itemset                        */
	int intersection = 0;                   /* local variable: computes the size of the intersection  */
	int sunion = 0;                         /* local variable: computes the size of the union         */
	for (i=0,j=0;(i<sizec)&&(j<sizee);)     /* scan the two itemsets and computes union and intersect,*/ 
    {
		if (representative[i]->id == e[j]->id)
        {     /* if the current item is equal  in both the itemsets     */ 
			intersection++;                 /* it belongs to their intersection                       */
			i++;j++;                        /* so that we can skip to the next item in both itemsets  */
		}
		else                                /* otherwise, just assuming that itemsets are ordered     */
			//if (representative[i] < e[j])   /* assume you are computing an ordered union              */
			if (representative[i]->id < e[j]->id)   /* assume you are computing an ordered union              */
				i++;                        /* incrementing either the left counter, if less          */ 
			else j++;                       /* or otherwise the right counter                         */
			sunion++;                       /* in each case, the current element is in the union      */
	}
	/* adjust the size of the union in order to include       */
	sunion += (sizec - i) + (sizee - j);    /* the elements eventually not considered previously      */
	if(sunion != 0)                         /* if the two itemsets are not empty                      */
		return (double)(1.0-(double)intersection/sunion);/* the distance is given by (union-intersection)/union   */
	else  return 0.0;                       /* otherwise the two itemsets are equivalent              */
}


void ClusterGamma::update(double gamma){         /* updates the cluster representative having adequate frq */
	representative.clear();                 /* first of all, the representative vector is reset       */
	Cluster::iterator end = this->end();    /* and the borders of the cluster container are computed  */
	double i=0;
	for (Cluster::iterator it = this->begin(); it != end; it++)
	{                                       /* for each element actually assigned to the cluster      */
		i=((double)(*it).second/population);
		if (i >= gamma)
		{
			ITEM *item = new ITEM;
			item->id = (*it).first;
			item->frq = i;

			//representative.push_back((*it).first);/* it belongs to the cluster representative */
			representative.push_back(item);/* it belongs to the cluster representative */
		}

//		cout << (*it).first << "\t" << (*it).second << "\tfreq = " << i << "\t" << fp->lookup((*it).first) << endl;

	}
	representative.prepare();               /* reorder the representative itemset                     */

	/*********
	cout << "----- dopo la prepare()" << endl;
	for (int j = 0; j < representative.size(); j++) {
		//cout << representative[j]->id << "\t" << " " << "\tfreq = " << frequence[j] << "\t" << fp->lookup(representative[j]->id) << endl;
		cout << representative[j]->id << "\t" << " " << "\tfreq = " << representative[j]->frq << "\t" << fp->lookup(representative[j]->id) << endl;
	}
	cout << "-----" << endl;
	/*********/
	
	clear();                                /* remove each assignment to the cluster       */
	Cluster::update(gamma);
}

void ClusterGamma::trash_update(){         /* updates the cluster representative having adequate frq */
	representative.clear();                 /* first of all, the representative vector is reset       */
	act_population = population;            /* record the population                                  */
	population = 0;                         /* and reset the actual counter                           */
}


//ostream& operator<<(ostream& s, ClusterGamma& c){/* a simple utility to print the cluster representative   */
///*	double t=c.topten()/100;*/
//	s << "<representative>\n";
//	for (int i = 0, size = c.representative.size(); i < size; i++)
//		/*if (c.frequence[i]>=t)*/
//			s << "\t\t\t<item id=\"" << c.fp->lookup(c.representative[i]) << "\"\tfrequency=\"" << c.frequence[i] << "\"> </item>\n" ;;
//	s << "\t\t</representative>" << /* c.topten() <<*/ endl;
//	
//	return s; 
//};

std::ostream& ClusterGamma::print(std::ostream& os, int n)
{
	if (!orderOutput) 
		representative.outputsort();

	if (XMLFormat) {
		// XML Version
		os << "<representative>\n";
		for (int i = 0, size = representative.size(); i < size; i++)
			/*if (c.frequence[i]>=t)*/
			//os << "\t\t\t<item id=\"" << fp->lookup(representative[i]->id) << "\"\tfrequency=\"" << frequence[i] << "\"> </item>\n" ;;
			os << "\t\t\t<item id=\"" << fp->lookup(representative[i]->id) << "\"\tfrequency=\"" << representative[i]->frq << "\"> </item>\n" ;;
		os << "\t\t</representative>" << /* c.topten() <<*/ std::endl;
	} else {
		// HTML Version
		if (n <= 0) 
			n = representative.size();
		if (representative.size() > 0) {
			for (int i = 0, size = representative.size(); i < size && i < n; i++) {
				double fab = representative[i]->frq;
				if (fab >= 0.8 && act_population > 1) 
					os << "            <b><font color=red>" <<  fp->lookup(representative[i]->id) << "</font></b>";
					//os << "            <b><font color=red>" <<  fp->lookup(representative[i]) << "</font></b>";
				else
					os << "            " << fp->lookup(representative[i]->id);
					//os << "            " << fp->lookup(representative[i]);
				os << "&nbsp;<font size=\"-2\">(" << (fab * 100) << "%)</font>";
				if (i < (n - 1) && i < (size - 1)) os << ", ";  
				os << std::endl;
			}
		} else 
			os << "            <em>Not Classified</em>" << std::endl;
	}
	
	return os; 
};

double ClusterGamma::setcoherence(ITEMSET &b) /* compute the coherence of the itemset with a given transaction */
{
		int i, j,                               /* itemset indexes                                        */
		sizec = representative.size(),      /* the size of the cluster representative                 */
		sizee = b.size();                   /* the size of the current itemset                        */
	int intersection = 0;                   /* local variable: computes the size of the intersection  */
	for (i=0,j=0;(i<sizec)&&(j<sizee);){/* scan the two itemsets and computes union and intersect,*/ 
		if (representative[i] == b[j]){     /* if the current item is equal  in both the itemsets     */ 
			intersection++;                 /* it belongs to their intersection                       */
			i++;j++;                        /* so that we can skip to the next item in both itemsets  */
		}
		else                                /* otherwise, just assuming that itemsets are ordered     */
			if (representative[i] < b[j])   /* assume you are computing an ordered union              */
				i++;                        /* incrementing either the left counter, if less          */ 
			else j++;                       /* or otherwise the right counter                         */

	}
	/* adjust the size of the union in order to include       */
	if(sizec != 0)                         /* if the two itemsets are not empty                      */
		return (double)((double)intersection/sizec);/* the distance is given by (union-intersection)/union   */
	else  return 0.0;                       /* otherwise the two itemsets are equivalent              */


}

