#include "Ctfscan.h"


#define _POSIX_SOURCE

#include <cstdio>
#include <cstdlib>
#include <cassert>

#ifdef STORAGE
#include "storage.h"
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ctfscan::ctfscan()
{                               /* --- create a table file scanner */
  int    i;                     /* loop variable */
  char   *p;                    /* to traverse character flags */

  for (p = this->cflags + 256, i = 256; --i >= 0; )
    *--p = '\0';                /* clear and initialize */
  this->cflags['\n'] = TFS_RECSEP;    /* character flags */
  this->cflags['\t'] = this->cflags[' '] = TFS_BLANK|TFS_FLDSEP;
}  /* tfs_create() */


/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
/* --- functions --- */
#define isblank(c)   tfs_istype(TFS_BLANK,  c)
#define isfldsep(c)  tfs_istype(TFS_FLDSEP, c)
#define isrecsep(c)  tfs_istype(TFS_RECSEP, c)
#define issep(c)     tfs_istype(TFS_FLDSEP|TFS_RECSEP, c)

/*----------------------------------------------------------------------
  Functions
----------------------------------------------------------------------*/

ctfscan* ctfscan::tfs_dup ()
{                               /* --- duplicate a table file scanner */
  ctfscan *dup;                  /* created duplicate */

  dup = new ctfscan;
  if (!dup) 
	  return NULL;        /* create a new table file scanner */
  dup->tfs_copy(this);           /* and copy source into it */
  return dup;                   /* return created duplicate */
}  /* tfs_dup() */

/*--------------------------------------------------------------------*/

void ctfscan::tfs_copy (const ctfscan *src)
{                               /* --- copy a table file scanner */
  int  i;                       /* loop variable */
  char *d; const char *s;       /* to traverse the character flags */

  assert(src);           /* check arguments */
  s = src->cflags +256; d = this->cflags +256;
  for (i = 256; --i >= 0; ) *--d = *--s;
}  /* tfs_copy() */             /* copy character flags */

/*--------------------------------------------------------------------*/

int ctfscan::tfs_sgetc (const char *s)
{                               /* --- get character from string */
  static const char *p = "";    /* to traverse string */
  int    c;                     /* character code */

  if (s) p = s;                 /* if new string given, set string */
  if (*p == '\0') return -1;    /* if at end of string, abort */
  if (*p != '\\') return (unsigned char)*p++;
  p++;                          /* skip '\' */
  switch ((unsigned char)*p++){ /* evaluate character after '\' */
    case 'a': return '\a';      /* 0x07 (BEL) */
    case 'b': return '\b';      /* 0x08 (BS)  */
    case 'f': return '\f';      /* 0x0c (FF)  */
    case 'n': return '\n';      /* 0x0a (NL)  */
    case 'r': return '\r';      /* 0x0d (CR)  */
    case 't': return '\t';      /* 0x09 (HT)  */
    case 'v': return '\v';      /* 0x0b (VT)  */
    case '0': case '1': case '2': case '3':
    case '4': case '5': case '6': case '7':
      c = *p++ -'0';            /* get octal character code */
      if ((*p >= '0') && (*p <= '7')) c = c *8 +*p++ -'0';
      else return c;            /* get second digit */
      if ((*p >= '0') && (*p <= '7')) c = c *8 +*p++ -'0';
      else return c;            /* get third digit */
      if (c >= 256) return 192 +(c % 64);
      else return c;            /* check range and return character */
    case 'x': p++;              /* get hexadecimal character code */
      if      ((*p >= '0') && (*p <= '9')) c = *p++ -'0';
      else if ((*p >= 'a') && (*p <= 'f')) c = *p++ -'a' +10;
      else if ((*p >= 'A') && (*p <= 'F')) c = *p++ -'A' +10;
      else return 'x';          /* get first digit */
      if      ((*p >= '0') && (*p <= '9')) c = c *16 +*p++ -'0';
      else if ((*p >= 'a') && (*p <= 'f')) c = c *16 +*p++ -'a' +10;
      else if ((*p >= 'A') && (*p <= 'F')) c = c *16 +*p++ -'A' +10;
      return c;                 /* get second digit and return char. */
    default:                    /* non-function characters */
      if (*p == '\0') return '\\';
      else            return (unsigned char)*p++;
  }                             /* return character or backslash */
}  /* tfs_sgetc() */

/*--------------------------------------------------------------------*/

int ctfscan::tfs_chars (int type, const char *chars)
{                               /* --- set characters */
  int  i, c, d;                 /* loop variable, characters */
  char *p;                      /* to traverse character flags */

  if (!chars) return -1;        /* if no characters given, abort */
  p = this->cflags +256;         /* clear character flags in type */
  for (i = 256; --i >= 0; ) *--p &= (char)~type;
  for (c = d = this->tfs_sgetc(chars); c >= 0; c = this->tfs_sgetc(NULL))
    this->cflags[c] |= (char)type;  /* set character flags */
  return (d >= 0) ? d : 0;      /* return first character */
}  /* tfs_chars() */

/*--------------------------------------------------------------------*/

int ctfscan::tfs_getfld (FILE* file, char *buf, int len,int *read)
{                               /* --- read a table field */
  int  c;                       /* character read */
  int  d;                       /* delimiter type */
  char *p;                      /* to traverse the buffer */

  assert(file && buf && (len >= 0));
  p = buf; *p = '\0';           /* clear read buffer */
  do {                          /* --- skip leading blanks */
    c = getc(file);             /* get next character */
    if (c == EOF) return (ferror(file)) ? -1 : TFS_EOF;
  } while (isblank(c));         /* while character is blank */
  if (issep(c)) {               /* check for field/record separator */
    if (read) *read = 0;        /* clear number of characters read */
    return (isfldsep(c)) ? TFS_FLD : TFS_REC;
  }                             /* return delimiter type */
  while (1) {                   /* --- read value */
    if (len >= 0) {             /* if buffer is not full, */
      len--; *p++ = (char)c; }  /* store character in buffer */
    c = getc(file);             /* get next character */
    if (issep(c)) { d = (isfldsep(c))  ? TFS_FLD : TFS_REC; break; }
    if (c == EOF) { d = (ferror(file)) ? -1      : TFS_EOF; break; }
  }                             /* while character is no separator */
  while (isblank(*--p));        /* --- remove trailing blanks */
  *++p = '\0';                  /* terminate string in buffer */
  if (read) *read = (int)(p -buf); /* store number of characters read */
  if (d != TFS_FLD) return d;   /* if not at field separator, abort */
  while (isblank(c)) {          /* --- skip trailing blanks */
    c = getc(file);             /* get next character */
    if (c == EOF) return (ferror(file)) ? -1 : TFS_EOF;
  }                             /* check for end of file */
  if (isrecsep(c))  return TFS_REC;  /* check for record separator */
  if (!isfldsep(c)) ungetc(c, file); /* put back character */
  return TFS_FLD;               /* return delimiter type */
}  /* tfs_getfld() */

