#include "FastVector.h"

struct idOrd : public std::binary_function<ITEM*, ITEM*, bool> {
	bool operator()(ITEM* x, ITEM* y) { return x->id < y->id; }
};

struct frqOrd : public std::binary_function<ITEM*, ITEM*, bool> {
	bool operator()(ITEM* x, ITEM* y) { return x->frq > y->frq; }
};

void ITEMSET::outputsort()
{
    /* --- sort set */	
	if (this->size() <= 1) return;                /* if less than two items, abort */
	//sort(this->begin(),this->end());              /* sort items in set and         */
	std::sort(this->begin(),this->end(), frqOrd());    /* sort items in set             */
}

void ITEMSET::prepare()
{
    /* --- sort set and remove duplicates */
    //vector<int>::iterator src,dst,end;  /* to traverse the item vector        */
	std::vector<ITEM*>::iterator src,dst,end;  /* to traverse the item vector        */
	
	if (this->size() <= 1) return;      /* if less than two items, abort      */
	//sort(this->begin(),this->end());    /* sort items in set and              */
	std::sort(this->begin(),this->end(), idOrd());    /* sort items in set and              */
	
	dst = src = this->begin();			/* pointers to prev. and curr. value  */
	end = this->end();					/* for all elements until the end     */
	for (;src != end; src++)            /* main loop                          */
		if (*src != *dst)               /* if current and previous are equal  */
			*(++dst) = *src;            /* shift current into previous        */
	if (++dst != end)                   /* remove duplicate items             */
		this->erase(dst,end);           /* by erasing the remaining elements  */
}  /* prepare() */


double ITEMSET::distance(ITEMSET &b)
{
	int i, j,                               /* itemset indexes                                        */
	sizec = this->size(),      /* the size of the cluster representative                 */
	sizee = b.size();                   /* the size of the current itemset                        */
	int intersection = 0;                   /* local variable: computes the size of the intersection  */
	int sunion = 0; /* local variable: computes the size of the union         */
	for (i=0,j=0; (i<sizec) && (j<sizee); ) {/* scan the two itemsets and computes union and intersect,*/ 
//		if ((*this)[i] == b[j]){     /* if the current item is equal  in both the itemsets     */ 
		if ((*this)[i]->id == b[j]->id){     /* if the current item is equal  in both the itemsets     */ 
			intersection++;                 /* it belongs to their intersection                       */
			i++;j++;                        /* so that we can skip to the next item in both itemsets  */
		}
		else                                /* otherwise, just assuming that itemsets are ordered     */
//			if ((*this)[i] < b[j])   /* assume you are computing an ordered union              */
			if ((*this)[i]->id < b[j]->id)   /* assume you are computing an ordered union              */
				i++;                        /* incrementing either the left counter, if less          */ 
			else j++;                       /* or otherwise the right counter                         */
			sunion++;                       /* in each case, the current element is in the union      */
	}
	/* adjust the size of the union in order to include       */
	sunion += (sizec - i) + (sizee - j);    /* the elements eventually not considered previously      */
	if(sunion != 0)                         /* if the two itemsets are not empty                      */
		return (double)(1.0-(double)intersection/sunion);/* the distance is given by (union-intersection)/union   */
	else  return 0.0;                       /* otherwise the two itemsets are equivalent              */
}

