#pragma warning (disable: 4786)

#include <cstring>
#include <cstdlib>
#include <cassert>
#include "File.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

File::File(const char* filename, bool ld)
{
	fn_in = new char[strlen(filename)+1];
	std::strcpy(fn_in,filename);
      // file = fopen(filename, "rb");


	load     = ld;
	maxcnt   = isetsz = 0;
	curr     = reccnt = 0;
//	bufsid   = -1;
}

File::~File()
{
	if (file)
//		std::fclose(file);
	if (fn_in)
		delete[] fn_in;
}

bool File::read(ITEMSET& s)
{
	if (curr == isetsz) {               /* if all records read               */
		curr = 0;                       /* reset position of current pointer */
		if (!load)                      /* if to work on input file          */
			if(!std::fseek(file, pos, SEEK_SET))
				error(E_FREAD,fn_in);   /* reset file position               */	
		return false;                   /* return no file read               */
	}

	if (load)                           /* if to work on memory              */
		cis = isets[curr];              /* get current itemset               */
	else                               
		get_iset();                     /* otherwhise read from file         */
	curr++;                             /* and increment the counter         */
	
	s = cis;                            /* return s                          */
	return true;
}


ITEMSET* File::read()
{
	if (curr == isetsz) {                    /* if all records read               */
		curr = 0;                            /* reset position of current pointer */
		if (!load)                           /* if to work on input file          */
			if(std::fseek(file, pos, SEEK_SET) != 0)
				error(E_FREAD,fn_in);        /* reset file position               */	
			return NULL;                     /* return no file read               */
	}
	
	if (load)                                /* if to work on memory              */
		cis = isets[curr];                   /* get current itemset               */
	else {                               
		get_iset();                          /* otherwhise read from file         */
		cis.prepare();                       /* sort and remove duplicates        */
	}
	curr++;                                  /* and increment the counter         */
	
	return &cis;                             /* return s                          */
	
}


int File::reorg(long thresh)
{
	SymTab tmap;                                 /* temporary symbol table        */
	int i, size;
	ITEM* item;
	std::vector<ITEM*> sorted = nimap.bvec;           /* bucket vector                 */
	std::sort(sorted.begin(),sorted.end(),frqcmp());  /* order by frequency            */

	for (i = 0, size = sorted.size(); 
	(i < size) && (sorted[i]->frq >= thresh); i++)
	{
		item = (ITEM*)tmap.insert(nimap.ids[sorted[i]->id], sizeof(ITEM));
		if (!item) 
			return E_NOMEM;                      /* add the new item to the map,  */
		item->frq = sorted[i]->frq;
	}
	nimap = tmap;

	return 0;
}

int File::reset()
{                                            /* resets to the beginning           */
	if (load)                                /* if to work on memory              */
		curr = 0;                            /* set the current pointer to 0      */
	else {                               
		if(std::fseek(file, pos, SEEK_SET) != 0)  /* otherwise reset the file pointer  */
			error(E_FREAD,fn_in);            /* checking for I/O errors           */	
	}
	return 0;                                /* return OK                         */
}

ITEMSET* File::read(long i)
{                                           /* read i-th record                   */
		if (i < isetsz) {                   /* if current record exist            */
			if (!load){                     /* if to work on input file           */
				int locpos = ftell(file);   /* get the local record position      */
				if(std::fseek(file, pos, SEEK_SET) != 0)  /* set the file pointer      */
					error(E_FREAD,fn_in);   /* checking for I/O errors            */	
				long cnt = 0;               /* get the local record counter       */
				while(read()&&(++pos<i));   /* read records in sequence until ith */
				if(fseek(file, locpos, SEEK_SET) != 0)  /* reset the file pointer */
					error(E_FREAD,fn_in);   /* checking for I/O errors            */	
			}
			else cis = isets[i];
			return &cis;                    /* return s                           */
		}
		else return NULL;                   /* return no file read                */
}

bool File::load_iset(int i, ITEMSET &trans)
{
	if (i < isetsz) {                   /* if current record exist            */
		if (!load){                     /* if to work on input file           */
			int locpos = ftell(file);   /* get the local record position      */
			if(std::fseek(file, pos, SEEK_SET) != 0)  /* set the file pointer      */
				error(E_FREAD,fn_in);   /* checking for I/O errors            */	
			long cnt = 0;               /* get the local record counter       */
			while(read()&&(++pos<i));   /* read records in sequence until ith */
			if(std::fseek(file, locpos, SEEK_SET) != 0)  /* reset the file pointer */
				error(E_FREAD,fn_in);   /* checking for I/O errors            */	
		} else
			trans = isets[i];

		return true;                    /* return s                           */

	} else
		return false;                   /* return no file read                */
	
	trans=isets[i];
}

char* File::readTrans(int trans)
{
	int l = 0;
       int i;
	for (i = 0; i < isets[trans].size(); i++) {
		l += std::strlen(lookup(isets[trans][i]->id)) + 1;
	}
	char* result = new char[l + 1];
	strcpy(result,"");
	for (i = 0; i < isets[trans].size(); i++) {
		std::strcat(result,lookup(isets[trans][i]->id));
		std::strcat(result," "); 
	}

	//fn_in = new char[strlen(filename)+1];
	//strcpy(fn_in,filename);
	return result;
}

