#ifndef FILE_HPP_INCLUDED 
#define FILE_HPP_INCLUDED 

#include <vector>
#include "resources.h"
#include "Ctfscan.h"
#include "SymTab.h"

#define BUFSIZE     256              /* size of read buffer */

class File  
{
public:
    char* readTrans(int trans);

	ITEMSET* read(long i);
	int reset();
	bool loaded() const;
	int items();
	
	hash_map<int, int> classcount;
	hash_map<std::string, int, hashString, eqstr> classes;  /* Transaction-class mapping */
	ITEMSET* read();
	SymTab& symbols();
	void insert_iset(ITEMSET is){isets.push_back(is);};
	bool load_iset(int i, ITEMSET &trans);
	const char* lookup(int id) const;
	const char* looktr(int id) const;
	int reorg(long thresh);
	int size() const;
	bool read(ITEMSET& s);           /* read an itemset from buffer        */
	File(const char* fname,                /* file name                          */
		bool load     = true         /* flag for loading item sets         */
);
	virtual ~File();
protected:
	FILE* file;                      /* file pointer                       */
	SymTab nimap;                    /* name/identifier map                */
	SymTab trmap;					 /* Transaction map					   */
private:
	virtual void read_blk() = 0;     /* caches the dataset into memory     */
	virtual int get_item () = 0;     /* --- read an item                   */
	virtual int get_iset () = 0;     /* --- read an item set               */
//	void prepare ();                 /* --- sort set and remove duplicates */
protected:
	std::vector<ITEMSET> isets;           /* item set vector                    */
	int				isetsz;			 /* number of itemsets                 */
	ITEMSET         cis;             /* current item set                   */
	int             reccnt;          /* number of records read             */
	bool            load;            /* flag for file-buffering            */
	int             maxcnt;          /* maximal number of items per set    */
	int             curr;            /* current record read                */
	int             pos;             /* starting reading position in file  */
	char*           fn_in;           /* file name                          */            
};



inline int File::items()
{
	return nimap.size();
}

inline bool File::loaded()
const{
	return load;
}

inline int File::size() const{                 /* get the number of itemset         */
	return isetsz;                             /* currently loaded                  */
}



inline const char* File::lookup(int id)        /* get the name of a given identifier*/
const{
	return nimap.lookup(id);

}

inline const char* File::looktr(int id)        /* get the name of a given transaction*/
const{
	return trmap.lookup(id);

}

inline SymTab& File::symbols()                  /* gets the symbol table             */
{
	return nimap;
}

#endif

