#include <ctime>
#include "Kmeans.h"
#include "AsciiFile.h"
#include "dist.h"


bool Kmeans::insert(ITEMSET& e, long pos, int n){ 
	bool flag = false;          /* local variable to check if the assigment changed */
	if (mapping[pos] != n)      /* if itemset pos wasn't previously assigned to n   */
		flag = true;            /* something changed sine the last cycle            */
	mapping[pos] = n;           /* the current cluster assigment for pos is n       */

	for (int i = 0, size = e.size(); i < size; i++) /* for each item in itemset e   */
	{
		//if (clusters[n]->find(e[i]) == clusters[n]->end()){
		if (clusters[n]->find(e[i]->id) == clusters[n]->end()){
								/* if the item is not in the cluster representative  */	
			//(*clusters[n])[e[i]] = 1;/* DEPRECATED: insert(hash_map<long,int>::value_type(e[i],1)); */
			(*clusters[n])[e[i]->id] = 1;/* DEPRECATED: insert(hash_map<long,int>::value_type(e[i],1)); */
		}                       /* insert the item into the cluster and set frq to 1 */
		else {
			//(*clusters[n])[e[i]]++;/* otherwise, update its frequency                   */
			(*clusters[n])[e[i]->id]++;/* otherwise, update its frequency                   */
		}
	}
	clusters[n]->population++;   /* the current population of the cluster increased  */
	return flag;                /* to indicate whether the cluster assignment changed  */
};

Kmeans::Kmeans(int k, long size,File* fp, bool XMLf) {   	/* initialize the clusters and mappping structs     */ 
	int i;     
	for (i = 0; i < k; i++){     /* k is the desired number of clusters             */
		Cluster* c;              /* To store the cluster to be created              */
		c = new ClusterGamma(XMLf);
		c->set_fileptr(fp);      /* set the local file, for item names resolution    */
		c->population = 0;
		clusters.push_back(c);  /* that are inserted into the clusters container    */
	}
	for (i = 0; i < size; i++)   /* size is the actual dimension of the database     */
		mapping[i] = -1;         /* the cluster assignment structure is initialized  */
	set_fileptr(fp);

	XMLFormat = XMLf;
}

Kmeans::~Kmeans(){
	int sz = clusters.size();
	for (int i = 0; i < sz; i++){
		delete clusters[i];
		clusters[i] = NULL;
	}
}


void Kmeans::setseeds(long sd)
{
	long pos = 0;                  /* local temporary variables                 */
	int clust = size()-1;          /* the number of cluster to be initialized   */
	int* seeds = new int[clust];   /* temporary variable to set the seeds       */
	UniformDist random(sd+INIT_SEED);  /* random number generator from uniform dist */
	long val = filep->size();          /* size of the dataset                       */
	ITEMSET* T;

	/* ------------------------------------------------------------------------ */
	/* ------- Set random cluster seeds --------------------------------------- */
	/* ------------------------------------------------------------------------ */
#ifndef QUIET                             /* if not quiet version               */
	clock_t t;                            /* use a timer                        */
	t = clock();                          /* start the timer                    */
	msg("setting the cluster seeds ... ");
#endif                                    /* print a log message                */
	for (int i = 0; i < clust; i++)
		seeds[i]= (int) (random()*val);	  /* get the recnum of the i-th seed    */
	for (int i = 0; i < this->size()-1; i++) {/* sort the seeds, using              */
		for (int j = 0; j < i; j++)       /* the simple bubblesort algorithm    */
			if (seeds[j] > seeds[i]) {
				int swap = seeds[j];
				seeds[j] = seeds[i];
				seeds[i] = swap;
			}
	}
	for (int i = 0; i < clust; i++)	{         /* read the  from the file            */
		if (T=filep->read(seeds[i]))      /* the itemset at position seeds[i]   */
			seed(pos++,*T);               /* and set it as cluster seed pos     */
	}
	/* ---------------------------------------------------------------------- */
	/* ------- Set the trash cluster ---------------------------------------- */
	/* ---------------------------------------------------------------------- */
	T->clear();                   /* empty the itemset                        */
	seed(pos,*T);                 /* and set it as the trash cluster          */
	if (seeds)                    /* if seeds have been allocated             */
	delete[] seeds;               /* delete the local variable pointing at    */
#ifndef QUIET                           /* if not quiet version               */
	msg("[%d seed(s)] done ", this->size());
	msg("[%.2fs].\n", (clock()-t) /(double)CLOCKS_PER_SEC);
#endif
}

void Kmeans::cycle(double conf,int maxlen)
{                                /* Main cycle of the algorithm               */
	ITEMSET* T;                  /* local temporary variable                  */
	double gamma = conf/100;     /* the confindence in unit value             */
	long pos = 0;                /* local temporary variable                  */
	bool nochange;               /* to check for cluster stability            */
	int iteration = 0;           /* to count the number of iterations         */

	int i;
	#ifdef _DEBUG
	clock_t t1;
	#endif
	filep->reset();                                /* reset the file pointer      */
	/* 
	ciclo che rappresenta le iterazioni dell'algoritmo
	ripetuto finche' il risultato non si stabilizza 
	o il numero di iterazioni e' minore del massimo 
	*/
#ifndef QUIET                                  /* if not quiet version             */
		msg("Performing iteration ");
#endif                                         /* print a log message              */
	do {                                       /* iterate the algorithm            */        
#ifndef QUIET                                  /* if not quiet version             */
		msg(" %d ",iteration+1);
#endif                                         /* print a log message              */
		nochange = false;                      /* by default, no change happened   */
		pos = 0;                               
		/* main cycle: assign each itemset to a cluster                            */
#ifdef _DEBUG                                  /* if not quiet version             */
		    msg("\ncomputing cluster assignments ... ");
			t1 = clock();                       /* start the timer                 */
#endif                                         /* print a log message              */
		while (T = filep->read()) {                /* for each itemset read           */
			double min = 2;                    /* set a huge value for the distance,      */
			double temp;                       /* a local variable for the distance val   */
			int ncluster;                      /* and for the cluster assignment          */
			for (i = 0; i < size(); i++)  /* compare each cluster representative          */
			{
				if ((temp = (*this)[i].distance(*T)) <= min)
				{                              /* if its distance is minimal              */
					min = temp;                /* assign the value of the distance        */
					ncluster = i;              /* and record the cluster representative   */
				}
			}
			nochange = insert(*T,pos,ncluster) || nochange;
			pos++;                         /* assign the itemset to the cluster,      */
		}		                           /* checking whether the assignment changed */
#ifdef _DEBUG                              /* if not quiet version                    */
	msg(" done [%.2fs].\n", (clock()-t1) /(double)CLOCKS_PER_SEC);  
    msg("recomputing cluster centers ... ");
	t1 = clock();                               /* start the timer */
#endif

//		cout << endl;
//		cout << "Iteration " << iteration << endl;
		for (i = 0; i < size()-1; i++) {     /* recompute cluster representatives            */
//			cout << "Cluster " << i << "\tpopulation " << (*this)[i].population << endl;
			(*this)[i].update(gamma);
		}
		(*this)[size()-1].trash_update();
#ifdef _DEBUG                                  /* if not quiet version                    */
	msg(" done [%.2fs].\n", (clock()-t1) /(double)CLOCKS_PER_SEC);
	msg("Performing iteration ");
#endif
	}		
	while ((nochange) && (++iteration < maxlen));

}

void Kmeans::dump(std::ostream& os, int pos)
{
	ITEMSET* T;                  /* temporary variables                    */
	long i,j,
		id = 1,                  /* the id of the transaction              */
		size = filep->size();       /* size of the dataset                    */
	for (i = 0; i < size; i++) { /* for each itemset                       */
		if (mapping[i] == pos){  /* test whether it belongs to cluster pos */
			T = filep->read(i);     /* in such case, read the dataset         */
			if (T != NULL)           /* and print it in relational format      */
				for (j = 0; j < T->size(); j++)
					//os << id << "\t" << filep->lookup((*T)[j]) << endl;
					os << id << "\t" << filep->lookup((*T)[j]->id) << std::endl;
		}
		id++;                    /* using the proper itemset identifier    */
	}
}

std::ostream& Kmeans::printRep(std::ostream& os, int index, int n) {
	// HTML version
	if (clusters[index]->act_population > 0)
		(*clusters[index]).print(os,n); 
	
	return os;
}

