#ifndef KMEANS_HPP_INCLUDED
#define KMEANS_HPP_INCLUDED

#include <iostream>
#include <fstream>
#include "AsciiFile.h"
#include "FastVector.h"
#include "ClusterGamma.h"

class Kmeans                         /* a container for all the clusters                */
{
	friend std::ostream& operator<<(std::ostream& s, Kmeans& k) {
		if (k.XMLFormat) {
			// XML Version
			for (int i = 0; i < k.clusters.size(); i++){
				s << "\t<cluster id=\"" << i
					<< "\" population=\"" << k.clusters[i]->act_population 
					<< "\">\n" << "\t\t";
				(*k.clusters[i]).print(s,0); 
				s << std::endl << "\t</cluster>" << std::endl;
			}
		} else {
			// HTML Version
			for (int i = 0; i < k.clusters.size(); i++){
				if (k.clusters[i]->act_population > 0) {
					s << "        <tr>" << std::endl;
					s << "          <td width=10% align=center bgColor=#eeeeee>" << std::endl;
					s << "            <a href=\"#" << i << "\">" << i << "</a>" << std::endl;
					s << "          </td>" << std::endl;
					s << "          <td width=10% align=center bgColor=#eeeeee>" << std::endl;
					s << "            <font color=green><b>" << k.clusters[i]->act_population << "</b></font>" << std::endl;
					s << "          </td>" << std::endl;
					s << "          <td bgColor=#ffeeee width=80%>" << std::endl;
					(*k.clusters[i]).print(s,5); 
					s << "          </td>" << std::endl;
					s << "        </tr>" << std::endl;
				}
			}
		}
		return s; 
	}	
public:
	File* filep; 
	virtual ~Kmeans();            /* by default, no operation is performed            */
protected:
	std::vector<Cluster*> clusters;
	hash_map<long, int,	hash<long> > mapping; /* for each tuple in the original dataset,          */
	      /* contains the assigment of the tuple to a cluster */		
	bool XMLFormat;
public:
	std::ostream& printRep(std::ostream& os, int index, int n = 0); 
	void cycle(double conf, int maxlen);
	void setseeds(long sd = 0);
	void dump(std::ostream& os, int pos);
	int retmapping(int i)
		{ return mapping[i]; };
	inline void set_fileptr(File* f){      /* sets the cluster representative                        */
		filep = f;                     /* with a simple assignment                               */
	};
	Kmeans(int k, long size,File* fp, bool XMLf);	/* initialize the clusters and mappping structs     */ 
	inline Cluster& operator[](int i) {
		return *clusters[i];
	}
	inline int size() {
		return clusters.size();
	}
	inline int clusterpop(int i) {
		return clusters[i]->act_population;
	};
	inline void seed(int pos, ITEMSET& t){
		clusters[pos]->set_representative(t);
	}
	bool insert(ITEMSET& e, long pos, int n);
};


#endif

