#include "Result.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Result::Result(Kmeans* a,bool mp, bool XMLf)
: mapping(mp), XMLFormat(XMLf)
{
	algo = a;                     /* Let us record the algorithm results */
}


std::ostream& Result::print(std::ostream& os,int iteration)
{
	if (XMLFormat) {
		if (iteration==0)
		{
			os << getDTS() << std::endl;            /* prints the DTS                       */
			os << "<experiment>\n";
			os << "<clustering>\n";
		}
		else 
		{
			os << "<refinement id=\"" << iteration <<"\">\n";
		};
			
		/* each output contains three elements  */
		/* the clusters                         */
		/* the mappings                         */
		/* the quality measures                 */
		/* --------------- print the cluster representatives ----------------------- */
		os << *(algo) << std::endl;   
		/* --------------- print the cluster mappings ------------------------------ */                                     
		if(mapping){
			os << "<mapping>\n";      
			
			for (int j=0; j < (algo->filep)->size(); j++){
				std::string identifier=(algo->filep)->looktr(j) ;
				os << "\t<assignement>\n" << "\t\t<tuple id=\"" << identifier  
					<< "\"/>\n" 
					<< "\t\t<clusterref id=\"" 
					<< algo->retmapping(j) 
					<< "\"/>\n" 
//					<< "\t\t<class id=\"" 
//					<< classi[pippo] << "\"/>\n" << 
					"\n\t</assignement>\n";
		}
			os << "</mapping>\n";
		}

		if (iteration==0)
			os << "</clustering>\n";
		else
			os << "</refinement>\n";
		return os;
	} else {
		// HTML version
		if (iteration==0) {
			os << "<html>" << std::endl;
			os << "<head>" << std::endl;
			os << "  <title>!!! TEST !!!</title>" << std::endl;
			os << "  <STYLE TYPE=\"text/css\" MEDIA=screen>" << std::endl;
			//os << "  <!--" << endl;
			os << "    BODY      { BACKGROUND: #ffffff fixed no-repeat 0px 0px; COLOR: #171717; FONT-FAMILY: Arial, Helvetica, sans-serif; MARGIN: 0pt }" << std::endl;
			os << "    A         { BACKGROUND-COLOR: transparent; COLOR: #336699; TEXT-DECORATION: none }" << std::endl;
			os << "    A:link    { BACKGROUND-COLOR: transparent; COLOR: #336699; TEXT-DECORATION: none }" << std::endl;
			os << "    A:visited { BACKGROUND-COLOR: transparent; COLOR: #336699 }" << std::endl;
			os << "    A:hover   { BACKGROUND-COLOR: transparent; COLOR: red; TEXT-DECORATION: underline }" << std::endl;
			os << "    FORM      { COLOR: #171717; FONT-FAMILY: Arial, Helvetica, sans-serif; FONT-SIZE: 8pt; FONT-WEIGHT: normal }" << std::endl;
			os << "    INPUT     { COLOR: #171717; FONT-FAMILY: Arial, Helvetica, sans-serif; FONT-SIZE: 8pt; FONT-WEIGHT: normal }" << std::endl;
			os << "    SELECT    { COLOR: #171717; FONT-FAMILY: Arial, Helvetica, sans-serif; FONT-SIZE: 8pt; FONT-WEIGHT: normal }" << std::endl;
			os << "    OPTION    { COLOR: #171717; FONT-FAMILY: Arial, Helvetica, sans-serif; FONT-SIZE: 8pt; FONT-WEIGHT: normal }" << std::endl;
			//os << "  -->" << endl;
			os << "  </STYLE>" << std::endl;
			os << "</head>" << std::endl;
			os << "<script>" << std::endl;
			os << "<!--" << std::endl;
			os << "function sf() {document.form.query.focus();}" << std::endl;
			os << "function help() { msgWindow=window.open (\"$helpPage\",\"displayWindow\",\"toolbar=no,resizable=yes,scrollbars=yes,width=600,height=600\") }" << std::endl;;
			os << "-->" << std::endl;
			os << "</script>"  << std::endl;
			os << "<body>" << std::endl;
			os << "<center>" << std::endl;
			os << "<img src=\"../images/WebCat.gif\" width=\"180\" alt=\"An Automatic Categorizator of Web Search Results\">" << std::endl;
			os << "<FORM METHOD=\"GET\" ACTION=\"$formAction\" name=form>" << std::endl;
			os << "<TABLE border=0 cellPadding=0 cellSpacing=0>" << std::endl;
			os << "  <tr>" << std::endl;
			os << "    <td align=\"center\">" << std::endl;
			os << "      <table>" << std::endl;
			os << "        <tr>" << std::endl;
			os << "          <td valign=\"middle\">" << std::endl;
			os << "            <img src=\"../images/WebCatName.gif\" width=\"120\">" << std::endl;
			os << "          </td>" << std::endl;
			os << "          <td valign=\"middle\">" << std::endl;
			os << "            <font face=\"verdana\">: An Automatic Categorizator of Web Search Results</font>" << std::endl;
			os << "          </td>" << std::endl;
			os << "        </tr>" << std::endl;
			os << "      </table>" << std::endl;
			os << "    </td>" << std::endl;
			os << "  </tr>" << std::endl;
			os << "  <TR vAlign=center>" << std::endl;
			os << "    <TD align=center>" << std::endl;
			os << "      <TABLE><TR>" << std::endl;
			os << "      <TD>" << std::endl;
			os << "         query" << std::endl;
			os << "      </TD>" << std::endl;
			os << "      <TD>" << std::endl;
			os << "         <INPUT value=\"@query\" maxLength=256 name=query size=55>" << std::endl;
			os << "      </TD>" << std::endl;
			os << "      <TD>" << std::endl;
			os << "        <FONT SIZE=\"-2\"><A HREF=\"javascript:help()\">help</A></FONT>" << std::endl;
			os << "      </TD>" << std::endl;
			os << "      </TR></TABLE>" << std::endl;
			os << "    </TD>" << std::endl;
			os << "  </TR>" << std::endl;
			os << "  <TR vAlign=center>" << std::endl;
			os << "    <TD>" << std::endl;
			os << "      <TABLE width=100% border=0>" << std::endl;
			os << "      <TR>" << std::endl;
			os << "      <TD>" << std::endl;
			os << "         number of clusters" << std::endl;
			os << "      </TD>" << std::endl;
			os << "      <TD align=middle>" << std::endl;
			os << "         <SELECT NAME=\"cluster\">" << std::endl;
    
			for (int i = 5; i < 35; i = i + 5) {
				if (i == algo->size()) {
	    			os << "           <OPTION SELECTED>  " << i << std::endl;
				  } else {
    				os << "           <OPTION>    " << i << std::endl;
				  }
			}

			os << "         </SELECT>" << std::endl;
			os << "      </TD>" << std::endl;
			os << "      <TD align=right>" << std::endl;
			os << "         confidence" << std::endl;
			os << "      </TD>" << std::endl;
			os << "      <TD align=middle>" << std::endl;
			os << "         <SELECT NAME=\"gamma\">" << std::endl;
			
			//@conf = qw(5 10 15 20 25 30 40 50 60);
			//for $c (@conf) {
			//	if ($c == $gamma) {
			//		os << "         <OPTION SELECTED>  $c" << endl;
			//	} else {
			//		os << "         <OPTION>    $c" << endl;
			//	}
			//}
			os << "           <OPTION>     5" << std::endl;
			os << "           <OPTION>    10" << std::endl;
			os << "           <OPTION>    15" << std::endl;
			os << "           <OPTION>    20" << std::endl;
			os << "           <OPTION>    25" << std::endl;
			os << "           <OPTION>    30" << std::endl;
			os << "           <OPTION>    40" << std::endl;
			os << "           <OPTION>    50" << std::endl;
			os << "           <OPTION>    60" << std::endl;

			os << "         </SELECT>" << std::endl;
			os << "      </TD>" << std::endl;
			os << "      <TD align=right>" << std::endl;
			os << "         number of results" << std::endl;
			os << "      </TD>" << std::endl;
			os << "      <TD align=middle>" << std::endl;
			os << "         <SELECT NAME=\"nres\">" << std::endl;
			
			//@numResult = qw(10 20 30 50 100 150 200);
			//for $n (@numResult) {
			//	if ($n == $nRes) {
			//		os << "         <OPTION SELECTED>  $n" << endl;
			//	} else {
			//		os << "         <OPTION>    $n" << endl;
			//	}
			//}
			os << "           <OPTION>    10" << std::endl;
			os << "           <OPTION>    20" << std::endl;
			os << "           <OPTION>    30" << std::endl;
			os << "           <OPTION>    50" << std::endl;
			os << "           <OPTION>   100" << std::endl;
			os << "           <OPTION>   150" << std::endl;
			os << "           <OPTION>   200" << std::endl;
			
			os << "         </SELECT>" << std::endl;
			os << "      </TD>" << std::endl;
			os << "      </TR>" << std::endl;
			os << "      </TABLE>" << std::endl;
			os << "    </TD>" << std::endl;
			os << "  </TR>" << std::endl;
			os << "  <TR vAlign=center>" << std::endl;
			os << "    <TD align=middle>" << std::endl;
			os << "    &nbsp;" << std::endl;
			os << "    </TD>" << std::endl;
			os << "  </TR>" << std::endl;
			os << "  <TR vAlign=center>" << std::endl;
			os << "    <TD align=middle>" << std::endl;
			os << "       <INPUT type=submit value=\"Search with Google\">" << std::endl;
			os << "    </TD>" << std::endl;
			os << "  </TR>" << std::endl;
			os << "  <tr>" << std::endl;
			os << "  <td align=\"right\">" << std::endl;
			os << "  <font size=-2><b><a href=\"http://www-kdd.cnuce.cnr.it/\">&copy;Pisa KDD Laboratory</a></b></font>" << std::endl;
			os << "  </td>" << std::endl;
			os << "  </tr>" << std::endl;
			os << "</TABLE>" << std::endl;
			os << "</FORM>" << std::endl;
			os << "</center>" << std::endl;
			os << "<BR><BR><a name=\"Top\"></a><BR>" << std::endl;
			os << "<CENTER><H3>Results (" << (algo->filep->size() + 1) << ")</H3></CENTER>" << std::endl;;
			os << "<BR>" << std::endl;

		} else {
			os << "<refinement id=\"" << iteration <<"\">\n";
		};
			

		//###
		//### Summary Table
		//###
		os << "<table width=100%>" << std::endl;
		os << "  <tbody>" << std::endl;
		os << "  <tr>" << std::endl;
		os << "    <td width=15%>" << std::endl;
		os << "      &nbsp;" << std::endl;
		os << "    </td>" << std::endl;
		os << "    <td width=70%>" << std::endl;
		os << "      <table width=100% border=1>" << std::endl;
		os << "        <thead>" << std::endl;
		os << "          <tr>" << std::endl;
		os << "            <th scope=col>Name</th>" << std::endl;
		os << "            <th scope=col>Size</th>" << std::endl;
		os << "            <th scope=col>Description</th>" << std::endl;
		os << "          </tr>" << std::endl;
		os << "        </thead>" << std::endl;
		os << "        <tbody>" << std::endl;
		/* --------------- print the cluster representatives ----------------------- */
		os << *(algo) << std::endl;
		/* --------------- print the cluster mappings ------------------------------ */                                     
		os << "        </tbody>" << std::endl;
		os << "      </table>" << std::endl;
		os << "    </td>" << std::endl;
		os << "    <td width=15%>" << std::endl;
		os << "      &nbsp;" << std::endl;
		os << "    </td>" << std::endl;
		os << "  </tr>" << std::endl;
		os << "</table>" << std::endl;
		os << "<br><br>" << std::endl;
		

        //###
		//### Elenco dettagliato dei cluster
		//###

        // Mapping: per ogni Cluster si ha la lista dei suoi elementi
		typedef std::vector<int> itemlist;
		std::vector <itemlist> ClusterItems;
		for (int i = 0; i < algo->size(); i++) {
			itemlist dummy;
			ClusterItems.push_back(dummy);
		}
		for (int i = 0; i < (algo->filep)->size(); i++) {
			ClusterItems[algo->retmapping(i)].push_back(i);
		}


		os << "<table>" << std::endl;
		os << "<tr>" << std::endl;
		os << "  <td width=5%>&nbsp;</td>" << std::endl;
		os << "  <td width=85%>" << std::endl;
		os << "    <table border=0>" << std::endl;
		os << "    <tbody>" << std::endl;

        for (int i = 0; i < ClusterItems.size(); i++) {
			if (ClusterItems[i].size() > 0) {
				os << "    <tr>" << std::endl;
	    		os << "      <td>" << std::endl;
	    		os << "        <a name=\"" << i << "\"> </a><hr>" << std::endl;
	    		os << "      </td>" << std::endl;
	    		os << "    </tr>" << std::endl;
	    		os << "    <tr>" << std::endl;
	    		os << "      <td>" << std::endl;
	    		os << "        <table cellPadding=2 cellSpacing=2 width=100%>" << std::endl;
	    		os << "        <tbody>" << std::endl;
				os << "        <tr>" << std::endl;
				os << "          <td width=10% align=left bgColor=#ffeeee vAlign=top>" << std::endl;
				os << "            <a href=\"#Top\"><img border=0 src=\"../images/turn18.gif\"></a>" << std::endl;
				os << "            <b>" << i << "</b>&nbsp;&nbsp;<font size=\"-2\">( <font color=green><b>" << ClusterItems[i].size() << "</b></font> )</font>" << std::endl;
				os << "          </td>" << std::endl;
				os << "          <td width=90% align=left bgColor=#ffeeee vAlign=top>" << std::endl;
				
				if (i < (ClusterItems.size() - 1))
					algo->printRep(os,i); // stampa il rap[presentate in formato leggibile
				else
					os << "            <em>Not Classified</em>" << std::endl;
								
				os << "          </td>" << std::endl;
				os << "        </tr>" << std::endl;
				os << "        <tbody>" << std::endl;
				os << "        </table>" << std::endl;
				os << "      </td>" << std::endl;
				os << "    </tr>" << std::endl;
				os << "    <tr>" << std::endl;
				os << "      <td>" << std::endl;
				os << "        <table width=100%>" << std::endl;
				os << "        <tbody>" << std::endl;
				os << "        <tr>" << std::endl;
				os << "          <td width=10%>&nbsp;</td>" << std::endl;
				os << "          <td width=90%>" << std::endl;
				
				//os << "$result{$id}{label}";
				int size = ClusterItems[i].size();
				for (int j = 0; j < ClusterItems[i].size(); j++) {
					os << "            <TABLE cellPadding=2 cellSpacing=2 width=\"100%\">" << std::endl;
					os << "            <TBODY>" << std::endl;
					os << "            <TR>" << std::endl;
					if (j % 2 == 1) {
  						os << "              <TD align=left bgColor=#eeeeee vAlign=top width=\"3%\">" << std::endl;
					} else {
  						os << "              <TD align=left vAlign=top width=\"3%\">" << std::endl;
					}
					//os << "                <B>num&nbsp;&nbsp;</B>" << endl;
					os << "                <B>" << (algo->filep)->looktr(ClusterItems[i][j]) << "&nbsp;&nbsp;</B>" << std::endl;
					os << "              </TD>" << std::endl;
					if (j % 2 == 1) {
  						os << "              <TD align=left bgColor=#eeeeee vAlign=top width=\"97%\">" << std::endl;
					} else {
  						os << "              <TD align=left vAlign=top width=\"97%\">" << std::endl;
					}
					//os << "                <A target=\"_blank\" href=\"" << "url" << "\">" << "title " << (algo->filep)->looktr(ClusterItems[i][j]) << "</A><BR>" << endl;
					os << "                " << (algo->filep)->readTrans(ClusterItems[i][j]) << std::endl;
					//os << "                " << "snippet" << endl;
					//os << "                <SPAN class=links><BR>" << "url" << "</SPAN> <BR>" << std::endl;
					os << "              </TD>" << std::endl;
					os << "            </TR>" << std::endl;
					os << "            </TBODY>" << std::endl;
					os << "            </TABLE>" << std::endl;
				}
				
				os << "          </td>" << std::endl;
				os << "        </tr>" << std::endl;
				os << "        </tbody>" << std::endl;
				os << "        </table>" << std::endl;
				os << "      </td>" << std::endl;
				os << "    </tr>" << std::endl;
			}
		}
		
  
		os << "    <tr>" << std::endl;
		os << "      <td>" << std::endl;
		os << "        <hr>" << std::endl;
		os << "      </td>" << std::endl;
		os << "    </tr>" << std::endl;
		os << "    </tbody>" << std::endl;
		os << "    </table>" << std::endl;    
		os << "  </td>" << std::endl;
		os << "  <td width=10%>&nbsp;</td>" << std::endl;
		os << "</tr>" << std::endl;
		os << "</table>" << std::endl;
		os << "</body></html>" << std::endl;

		return os;
	}
}



//ostream& operator<<(ostream& os, Result& rs){
//	os << rs.getDTS() <<endl;            /* prints the DTS                       */
//									     /* each output contains three elements  */
//	                                     /* the clusters                         */
//	                                     /* the mappings                         */
//	                                     /* the quality measures                 */
//	/* --------------- print the cluster representatives ----------------------- */
//	os << *(rs.algo) << endl << flush;   
//	/* --------------- print the cluster mappings ------------------------------ */                                     
//	if(rs.mapping){
//		os << "<mapping>\n";      
//		
//		for (int j=0; j < ((rs.algo)->filep)->size(); j++){
//			string identifier=((rs.algo)->filep)->looktr(j) ;
//			os << "\t<assignement>\n" << "\t\t<tuple id=\"" << identifier  
//				<< "\"/>\n" 
//				<< "\t\t<clusterref id=\"" 
//				<< rs.algo->retmapping(j) 
//				<< "\"/>\n" 
//				<< "\t\t<class id=\"" 
//				<< classi[pippo] << "\"/>\n" << 
//				"\n\t</assignement>\n";
//	}
//		os << "</mapping>\n";
//	}
//	if ((rs.f_comp) || (rs.quality)){
//		os << "<qualitymeasures>\n";
//		if (rs.quality){
//			os << "\n<Q1>\t" << rs.getQ1() << " </Q1>\n\n"; 
//			os << "\n<Q2>\t" << rs.getQ2() << " </Q2>\n\n";  
//		}
//		if (rs.f_comp){
//			os << "\n<f-measure>\t" << rs.getF() << " </f-measure>\n\n" << flush; 
//
//		}		
//		os << "</qualitymeasures>\n";
//	}
//
//	os << "</clustering>\n";
//	os << "</experiment>\n";
//
//	return os;
//}


