// SymTab1.cpp: implementation of the SymTab class.
//
//////////////////////////////////////////////////////////////////////

#pragma warning (disable: 4786)


#include "SymTab.h"

#define _POSIX_SOURCE
#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <string.h>
#include <limits.h>
#include <assert.h>
#ifdef NIMAPFN
#include "vecops.h"
#endif

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
#define DFLT_INIT     1023      /* default initial hash table size */
#if (INT_MAX > 32767)
#define DFLT_MAX   1048575      /* default maximal hash table size */
#else
#define DFLT_MAX     16383      /* default maximal hash table size */
#endif
#define BLKSIZE        256      /* block size for identifier vector */



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////



SymTab::SymTab(int init, int max)
{                                   /* --- create a symbol table */
  if (init <= 0) init = DFLT_INIT;  /* check and adapt initial */
  if (max  <= 0) max  = DFLT_MAX;   /* and maximal vector size */
  

  this->level = 0;                  /* allocate bucket vector */

}  /* st_create() */


SymTab::~SymTab()
{
  _delsym();                 /* delete all symbols, */
}


/*----------------------------------------------------------------------
  Auxiliary Functions
----------------------------------------------------------------------*/

void SymTab::_delsym ()
{                               /* --- delete all symbols */
  int i;                        /* loop variable */
  ITEM *ste;                    /* to traverse the symbol list */

  for (i = bvec.size()-1; --i >= 0; ) {  /* traverse bucket vector */
    ste = bvec[i];	            /* get the next bucket list, */
    bvec[i] = NULL;             /* clear the bucket vector entry, */
    delete ste;                 /* deallocate the symbol table element */                   
  }                            
  bvec.clear();
  ids.clear();
  types.clear();
  hbkt.clear();
}  /* _delsym() */

/*--------------------------------------------------------------------*/


/*----------------------------------------------------------------------
  Symbol Table Functions
----------------------------------------------------------------------*/





void* SymTab::insert (const std::string& name, unsigned size, int type)
{								        /* --- insert a symbol */
	ITEM* ste;				           	/* new symbol table element */
//	int val = atoi(name);               /* used to code chars in integer */
	if ((hbkt.find(name) != hbkt.end()) /* if symbol found on current level */
		&& (types[hbkt[name]] == type))  
		return EXISTS;                  /* return 'symbol exists'    */

	ste = new ITEM;						/* allocate new item         */
	ste->id = bvec.size();				/* get id value              */
	bvec.push_back(ste);				/* insert into bucket vector */
	ids.push_back(name);				/* store name                */
	types.push_back(type);				/* store type				 */
	hbkt[name] = ste->id;				/* initialize hash table     */
	return ste;							/* return the item			 */
}  /* st_insert() */



int SymTab::remove (const std::string& name, int type)
{                                   /* --- remove a symbol/all symbols */
  int i;                            /* index of hash bucket */
  ITEM *ste;                        /* to traverse bucket list */

  /* --- remove all symbols --- */
//  if (!name) {                      /* if no symbol name given */
//    _delsym();                      /* delete all symbols */
//    this->level = 0;                /* reset visibility level */
//    return 0;                       /* and symbol counter */
//  }                                 /* and return 'ok' */
// int val = atoi(name);              /* used to code chars in integer */
 /* --- remove one symbol --- */
  if ((hbkt.find(name) == hbkt.end())    /* if the symbol does not exist,  */
	  || (types[hbkt[name]] != type))
	  return -1;                    /* abort the function */
  i = hbkt[name];                    /* compute index of hash bucket   */
  ste = bvec[i];                    /* get symbol */
  bvec[i] = NULL;                   /* remove symbol from hash bucket */
  types[i] = 0;                     /* reset type */
  ids[i] = "REMOVED";               /* reset identity assignment      */
  delete ste;
  hbkt.erase(hbkt.find(name));       /* and symbol table element       */
  return 0;                         /* return 'ok'                    */
}  /* st_remove() */



void* SymTab::lookup (const std::string& name, int type)
{                               /* --- look up a symbol */
	int i;                      /* index of hash bucket */
//	assert(name);                 /* check arguments */
//	int val = atoi(name);               /* used to code chars in integer */

	if (hbkt.find(name) != hbkt.end()) 
		i = hbkt[name];         /* compute index of hash bucket */
	
	else
		return 0;			/* return 'not found' */
		
	if (types[i] == type)       /* if symbol found */
		return bvec[i];			/* return pointer to assoc. data */
	else
		return 0;			/* return 'not found' */
}  /* st_lookup() */


const char* SymTab::lookup(int id) const
{
	if (id < ids.size())        /* rensembles the name of the file  */
		return ids[id].c_str(); /* and returns it as answer         */
	else 
		return "";//NULL;
}

int SymTab::lookfr(int id)
{
	if (id < bvec.size())        /* rensembles the name of the file  */
		return int(bvec[id][2].frq); /* and returns it as answer         */
	else 
		return 0;
}

int SymTab::size()
{                               /* returns the number of items coded   */
	return bvec.size();         /* that are contained in bucket vector */
}



SymTab& SymTab::operator =(const SymTab &s)
{
	_delsym();                  /* delete all the elements             */
	bvec  = s.bvec;             /* copy the bucket vector              */
	hbkt  = s.hbkt;             /* copy the hash table                 */
	ids   = s.ids;              /* copy the identifiers vector         */
	types = s.types;            /* copy the types vector               */
	level = s.level;            /* copy the level                      */
return *this;
}
