#ifndef SYMTAB_HPP_INCLUDED
#define SYMTAB_HPP_INCLUDED


#include <vector>
#include <string>   
#include "FastVector.h"
#include "platform/hashmap.h"


#define EXISTS  ((void*)-1)     /* symbol exists already */

class HashTree;

          
struct eqstr
{
	bool operator()(std::string s1, std::string s2) const
	{
		return (s1 == s2);
	}
};


class hashString
{
public:
	size_t operator()(const std::string &str) const
	{
		return hash<const char*>()( str.c_str() );
	}
};


class SymTab  
{
	friend class HashTree;
	friend class File;
public:
	SymTab(int init = 0, int max =  0);
	virtual ~SymTab();
protected:
  hash_map<std::string, int, hashString, eqstr> hbkt;
     			                     /* hash table with indexes */
  int              level;            /* current visibility level  */
  std::vector<ITEM*>    bvec;             /* bucket vector */
  std::vector<std::string>   ids;              /* identifier vector */
  std::vector<int>      types;			 /* types vector      */
public:
	int size();
	const char* lookup(int id) const;
	void*          insert (const std::string& name, unsigned size, int type = 0);
	int            remove (const std::string& name, int type = 0);
	void*          lookup (const std::string& name, int type = 0);
	int lookfr(int i);
#ifdef _DEBUG
	void      stats ();
#endif // _DEBUG
public:
	SymTab& operator =(const SymTab &s);
private:
	void     _delsym ();
};











/*----------------------------------------------------------------------
  Symbol Table Functions
----------------------------------------------------------------------*/
extern void        st_begblk  (SymTab *tab);
extern void        st_endblk  (SymTab *tab);
extern int         st_symcnt  (const SymTab *tab);
extern const char* st_name    (const void *data);
extern int         st_type    (const void *data);
#ifndef NDEBUG
extern void        st_stats   (const SymTab *tab);
#endif

/*----------------------------------------------------------------------
  Name/Identifier Map Functions
----------------------------------------------------------------------*/
#ifdef NIMAPFN
extern void        nim_delete (NIMAP *nim);
extern void*       nim_byid   (NIMAP *nim, int id);
extern const char* nim_name   (const void *data);
extern int         nim_cnt    (const NIMAP *nim);
extern void        nim_sort   (NIMAP *nim, SYMCMPFN cmpfn, void *data,
                               int *map, int dir);
#ifndef NDEBUG
extern void        nim_stats  (const NIMAP *nimap);
#endif
#endif
/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/




#endif
