/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    error.cpp
 *	  Contains the error codes and messages 
 *    Copyright (C) 2001 Cristian Gozzi,Giuseppe Manco
 *
 */


#include <resources.h>


void error (int code, ...)
{                               /* --- print an error message */
  #ifndef QUIET                 /* if not quiet version */
  va_list    args;              /* list of variable arguments */
  const char *msg;              /* error message */
  #endif  /* #ifndef QUIET */
	 char arglist[1024];
  if ((code > 0) || (code < E_UNKNOWN))
    code = E_UNKNOWN;           /* check error code */
  #ifndef QUIET                 /* if not quiet version */
  msg = errmsgs[-code];         /* get error message text */
//  fprintf(stderr, "\n%s: ", prgname);
  va_start(args, code);         /* get variable arguments */
  vfprintf(stderr, msg, args);  /* print the error message */
  vsprintf(arglist,msg,args); 
  va_end(args);                 /* end variable argument evaluation */
  #endif  /* #ifndef QUIET */

  exit(code);                   /* abort the program */
}  /* error() */

#ifndef QUIET

void msg (const char *fmt, ...)
{                               /* --- print a message to stderr */
  va_list args;                 /* list of variable arguments */  
  va_start(args, fmt);          /* get variable arguments, */
  vfprintf(stderr, fmt, args);  /* print the message, and */
  va_end(args);                 /* end argument evaluation */
}  /* msg() */

#endif
